//+------------------------------------------------------------------+
//|                                              ATRTrailingStop.mqh |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"

//Dependencies
#include  <Trade\Trade.mqh>
#include <Trade\OrderInfo.mqh>

CTrade trade;
class COrderInfo;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class ATRTrailingStop
  {
private:
   double            m_atr_multiple;
   double            min_volume,max_volume_increase, volume_step, buy_stop_loss, sell_stop_loss,ask, bid,atr_stop,mid_point,risk_equity;
   double            take_profit;
   double            moving_average_low_array[],close_average_reading[],moving_average_high_array[],atr_reading[];
   long              min_distance;
   int               ma_high,ma_low,angle_of_averages,atr,close_average,ticket_1,ticket_2;
   bool              authorized;
   double            tick_value,average_market_move,margin,mid_point_height,channel_width,lot_step,desired_atr_multiple;
   bool              all_closed;



public:
   void              check_atr_stop(void);
   void              Update(void);
   void              Init(double _atr_multiple);
                     ATRTrailingStop();
                    ~ATRTrailingStop();
  };


//Can we move the stop loss or the take profit?
void ATRTrailingStop::check_atr_stop(void)
  {

   for(int i = PositionsTotal() -1; i >= 0; i--)
     {

      string symbol = PositionGetSymbol(i);
      if(_Symbol == symbol)
        {

         ulong ticket = PositionGetInteger(POSITION_TICKET);
         double position_price = PositionGetDouble(POSITION_PRICE_OPEN);
         long type = PositionGetInteger(POSITION_TYPE);
         double current_stop_loss = PositionGetDouble(POSITION_SL);
         double current_take_profit = PositionGetDouble(POSITION_TP);
         if(type == POSITION_TYPE_BUY)
           {
            double atr_stop_loss = (ask - (atr_stop));
            double atr_take_profit = 1;
            if(current_take_profit == 0)
               {
                  atr_take_profit = (ask + (atr_stop));
               }
            else
               {
                  atr_take_profit = current_take_profit;
               }
            

            if((current_stop_loss < atr_stop_loss) || (current_stop_loss == 0))
              {
               trade.PositionModify(ticket,atr_stop_loss,atr_take_profit);
              }

           }

         else
            if(type == POSITION_TYPE_SELL)
              {
               double atr_stop_loss = (bid + (atr_stop));
               double atr_take_profit = 1;
             if(current_take_profit == 0)
               {
                  atr_take_profit = (bid - (atr_stop));
               }
            else
               {
                  atr_take_profit = current_take_profit;
               }
               if((current_stop_loss > atr_stop_loss) || (current_stop_loss == 0))
                 {
                  trade.PositionModify(ticket,atr_stop_loss,atr_take_profit);
                 }
              }
        }
     }
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ATRTrailingStop::Update(void)
  {
//Update the technical information
   ask = SymbolInfoDouble(_Symbol,SYMBOL_ASK);
   bid = SymbolInfoDouble(_Symbol,SYMBOL_BID);
   buy_stop_loss = 0;
   sell_stop_loss = 0;
   CopyBuffer(ma_high,0,0,1,moving_average_high_array);
   CopyBuffer(ma_low,0,0,1,moving_average_low_array);
   CopyBuffer(atr,0,0,1,atr_reading);
   ArraySetAsSeries(moving_average_high_array,true);
   ArraySetAsSeries(moving_average_low_array,true);
   ArraySetAsSeries(atr_reading,true);
   atr_stop = ((atr_reading[0]) * m_atr_multiple);

//Do we have positions to manage?
   if(PositionsTotal() > 0)
     {
      check_atr_stop();
     }

  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ATRTrailingStop::Init(double _atr_multiple)
  {
   m_atr_multiple = _atr_multiple;
   desired_atr_multiple = _atr_multiple;
   take_profit = 0;
   authorized = false;
   all_closed =true;
   atr = iATR(_Symbol,PERIOD_CURRENT,14);
   min_volume = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);
   max_volume_increase = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MAX) / SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);
   min_distance = SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
   tick_value = SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_VALUE_PROFIT) * min_volume;
   lot_step = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
ATRTrailingStop::ATRTrailingStop()
  {
   Print("Volatility Doctor ATR Stop Loss.");
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
ATRTrailingStop::~ATRTrailingStop()
  {
   m_atr_multiple = 0;
  }
//+------------------------------------------------------------------+
